@@ -298,6 +298,9 @@ g_daemon_vfs_init (GDaemonVfs *vfs)
 
   bindtextdomain (GETTEXT_PACKAGE, GVFS_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  
+  if (g_thread_supported ())
+    dbus_threads_init_default ();
 
   vfs->async_bus = dbus_bus_get_private (DBUS_BUS_SESSION, NULL);
 
@@ -306,9 +309,6 @@ g_daemon_vfs_init (GDaemonVfs *vfs)
 
   g_assert (the_vfs == NULL);
   the_vfs = vfs;
-  
-  if (g_thread_supported ())
-    dbus_threads_init_default ();
 
   /* We disable SIGPIPE globally. This is sort of bad
      for s library to do since its a global resource.
@@ -386,7 +386,6 @@ convert_fuse_path (GVfs     *vfs,
       if (mount_info)
 	{
 	  g_object_unref (file);
-	  /* TODO: Do we need to look at the prefix of the mount_spec? */
 	  file = g_daemon_file_new (mount_info->mount_spec, mount_path);
 	  g_free (mount_path);
 	  g_mount_info_unref (mount_info);
@@ -649,6 +648,7 @@ fill_mountable_info (GDaemonVfs *vfs)
 
   infos = g_ptr_array_new ();
   uri_schemes = g_ptr_array_new ();
+  g_ptr_array_add (uri_schemes, g_strdup ("file"));
   count = 0;
   do
     {
@@ -993,6 +993,7 @@ _g_daemon_vfs_get_mount_info_by_fuse_sync (const char *fuse_path,
   DBusMessageIter iter;
   DBusError derror;
   int len;
+  const char *mount_path_end;
 	
   info = lookup_mount_info_by_fuse_path_in_cache (fuse_path,
 						  mount_path);
@@ -1031,9 +1032,12 @@ _g_daemon_vfs_get_mount_info_by_fuse_sync (const char *fuse_path,
 	{
 	  len = strlen (info->fuse_mountpoint);
 	  if (fuse_path[len] == 0)
-	    *mount_path = g_strdup ("/");
+	    mount_path_end = "/";
 	  else
-	    *mount_path = g_strdup (fuse_path + len);
+	    mount_path_end = fuse_path + len;
+
+	  *mount_path = g_build_filename (info->mount_spec->mount_prefix,
+					  mount_path_end, NULL);
 	}
       else
 	{
@@ -1236,7 +1240,7 @@ _g_daemon_vfs_append_metadata_for_set (DBusMessage *message,
   res = 0;
   if (type == G_FILE_ATTRIBUTE_TYPE_STRING)
     {
-      const char *current;
+      char *current;
       const char *val = (char *)value;
 
       current = meta_tree_lookup_string (tree, path, key);
@@ -1248,6 +1252,7 @@ _g_daemon_vfs_append_metadata_for_set (DBusMessage *message,
 				       DBUS_TYPE_STRING, &val,
 				       0);
 	}
+      g_free (current);
     }
   else if (type == G_FILE_ATTRIBUTE_TYPE_STRINGV)
     {
@@ -1262,6 +1267,7 @@ _g_daemon_vfs_append_metadata_for_set (DBusMessage *message,
 				       DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &val, g_strv_length (val),
 				       0);
 	}
+      g_strfreev (current);
     }
   else if (type == G_FILE_ATTRIBUTE_TYPE_INVALID)
     {
@@ -1581,3 +1587,14 @@ void
 g_io_module_unload (GIOModule   *module)
 {
 }
+
+char **
+g_io_module_query (void)
+{
+  char *eps[] = {
+    G_VFS_EXTENSION_POINT_NAME,
+    G_VOLUME_MONITOR_EXTENSION_POINT_NAME,
+    NULL
+  };
+  return g_strdupv (eps);
+}
