@@ -297,7 +297,7 @@ archive_file_get_from_path (ArchiveFile *file, const char *filename, gboolean ad
 static void
 create_root_file (GVfsBackendArchive *ba)
 {
-  ArchiveFile *root = g_slice_new0 (ArchiveFile);
+  ArchiveFile *root;
   GFileInfo *info;
   char *s, *display_name;
   GIcon *icon;
@@ -590,6 +590,7 @@ do_unmount (GVfsBackend *backend,
   GVfsBackendArchive *ba = G_VFS_BACKEND_ARCHIVE (backend);
 
   g_object_unref (ba->file);
+  ba->file = NULL;
   archive_file_free (ba->files);
   ba->files = NULL;
 
