@@ -779,9 +779,10 @@ should_volume_be_ignored (GduPool *pool, GduVolume *volume, GList *fstab_mount_p
   const gchar *type;
 
   ret = TRUE;
-  device = NULL;
 
   device = gdu_presentable_get_device (GDU_PRESENTABLE (volume));
+  if (device == NULL)
+    goto out;
 
   if (gdu_device_get_presentation_hide (device))
     goto out;
@@ -827,7 +828,8 @@ should_volume_be_ignored (GduPool *pool, GduVolume *volume, GList *fstab_mount_p
 
  out:
 
-  g_object_unref (device);
+  if (device != NULL)
+    g_object_unref (device);
   return ret;
 }
 
@@ -842,7 +844,6 @@ should_drive_be_ignored (GduPool *pool, GduDrive *d, GList *fstab_mount_points)
   GList *l;
 
   ignored = FALSE;
-  device = NULL;
   enclosed = NULL;
 
   device = gdu_presentable_get_device (GDU_PRESENTABLE (d));
@@ -1060,26 +1061,62 @@ find_disc_volume_for_device_file (GGduVolumeMonitor *monitor,
                                   const gchar       *device_file)
 {
   GList *l;
+  GGduVolume *ret;
+  struct stat stat_buf;
+
+  ret = NULL;
+
+  if (stat (device_file, &stat_buf) == 0)
+    {
+      for (l = monitor->disc_volumes; l != NULL; l = l->next)
+        {
+          GGduVolume *volume = G_GDU_VOLUME (l->data);
+          if (g_gdu_volume_has_dev (volume, stat_buf.st_rdev))
+            {
+              ret = volume;
+              goto out;
+            }
+        }
+    }
 
   for (l = monitor->disc_volumes; l != NULL; l = l->next)
     {
       GGduVolume *volume = G_GDU_VOLUME (l->data);
 
       if (g_gdu_volume_has_device_file (volume, device_file))
-        return volume;
+        {
+          ret = volume;
+          goto out;
+        }
     }
 
-  return NULL;
+ out:
+  return ret;
 }
 
 static GGduVolume *
 find_volume_for_device_file (GGduVolumeMonitor *monitor,
-                            const gchar       *device_file)
+                             const gchar       *device_file)
 {
   GList *l;
   GGduVolume *ret;
+  struct stat stat_buf;
 
   ret = NULL;
+
+  if (stat (device_file, &stat_buf) == 0)
+    {
+      for (l = monitor->volumes; l != NULL; l = l->next)
+        {
+          GGduVolume *volume = G_GDU_VOLUME (l->data);
+          if (g_gdu_volume_has_dev (volume, stat_buf.st_rdev))
+            {
+              ret = volume;
+              goto out;
+            }
+        }
+    }
+
   for (l = monitor->volumes; l != NULL; l = l->next)
     {
       GGduVolume *volume = G_GDU_VOLUME (l->data);
@@ -1112,12 +1149,19 @@ find_drive_by_device_file (GGduVolumeMonitor *monitor,
                            const gchar       *device_file)
 {
   GList *l;
+  struct stat stat_buf;
+
+  if (stat (device_file, &stat_buf) != 0)
+    {
+      g_warning ("%s:%s: Error statting %s: %m", G_STRLOC, G_STRFUNC, device_file);
+      return NULL;
+    }
 
   for (l = monitor->drives; l != NULL; l = l->next)
     {
       GGduDrive *drive = G_GDU_DRIVE (l->data);
 
-      if (g_gdu_drive_has_device_file (drive, device_file))
+      if (g_gdu_drive_has_dev (drive, stat_buf.st_rdev))
         return drive;
     }
 
@@ -1183,9 +1227,6 @@ update_drives (GGduVolumeMonitor *monitor,
   for (l = removed; l != NULL; l = l->next)
     {
       GduPresentable *p = GDU_PRESENTABLE (l->data);
-      GduDevice *d;
-
-      d = gdu_presentable_get_device (p);
 
       drive = find_drive_by_presentable (monitor, p);
       if (drive != NULL)
@@ -1195,16 +1236,11 @@ update_drives (GGduVolumeMonitor *monitor,
           monitor->drives = g_list_remove (monitor->drives, drive);
           *removed_drives = g_list_prepend (*removed_drives, drive);
         }
-      if (d != NULL)
-        g_object_unref (d);
     }
 
   for (l = added; l != NULL; l = l->next)
     {
       GduPresentable *p = GDU_PRESENTABLE (l->data);
-      GduDevice *d;
-
-      d = gdu_presentable_get_device (p);
 
       drive = find_drive_by_presentable (monitor, p);
       if (drive == NULL)
@@ -1217,8 +1253,6 @@ update_drives (GGduVolumeMonitor *monitor,
               *added_drives = g_list_prepend (*added_drives, g_object_ref (drive));
             }
         }
-      if (d != NULL)
-        g_object_unref (d);
     }
 
   g_list_free (added);
@@ -1280,15 +1314,18 @@ update_volumes (GGduVolumeMonitor *monitor,
 
       d = gdu_presentable_get_device (p);
 
-      volume = find_volume_for_device_file (monitor, gdu_device_get_device_file (d));
-      if (volume != NULL)
+      if (d != NULL)
         {
-          /*g_debug ("removing volume %s", gdu_device_get_device_file (d));*/
-          g_gdu_volume_removed (volume);
-          monitor->volumes = g_list_remove (monitor->volumes, volume);
-          *removed_volumes = g_list_prepend (*removed_volumes, volume);
+          volume = find_volume_for_device_file (monitor, gdu_device_get_device_file (d));
+          if (volume != NULL)
+            {
+              /*g_debug ("removing volume %s", gdu_device_get_device_file (d));*/
+              g_gdu_volume_removed (volume);
+              monitor->volumes = g_list_remove (monitor->volumes, volume);
+              *removed_volumes = g_list_prepend (*removed_volumes, volume);
+            }
+          g_object_unref (d);
         }
-      g_object_unref (d);
     }
 
   for (l = added; l != NULL; l = l->next)
@@ -1296,25 +1333,44 @@ update_volumes (GGduVolumeMonitor *monitor,
       GduPresentable *p = GDU_PRESENTABLE (l->data);
       GduDevice *d;
 
+      volume = NULL;
       d = gdu_presentable_get_device (p);
 
-      volume = find_volume_for_device_file (monitor, gdu_device_get_device_file (d));
+      if (d != NULL)
+        volume = find_volume_for_device_file (monitor, gdu_device_get_device_file (d));
+
       if (volume == NULL)
         {
-          GduPresentable *toplevel_presentable;
+          GduPresentable *toplevel_drive;
 
-          toplevel_presentable = gdu_presentable_get_toplevel (p);
-          if (toplevel_presentable != NULL)
+          toplevel_drive = gdu_presentable_get_enclosing_presentable (p);
+          /* handle logical partitions enclosed by an extented partition */
+          if (GDU_IS_VOLUME (toplevel_drive))
             {
-              GduDevice *toplevel_device;
+              GduPresentable *temp;
+              temp = toplevel_drive;
+              toplevel_drive = gdu_presentable_get_enclosing_presentable (toplevel_drive);
+              g_object_unref (temp);
+            }
 
-              toplevel_device = gdu_presentable_get_device (toplevel_presentable);
-              drive = find_drive_by_device_file (monitor, gdu_device_get_device_file (toplevel_device));
-              /*g_debug ("adding volume %s (drive %s)",
-                       gdu_device_get_device_file (d),
-                       gdu_device_get_device_file (toplevel_device));*/
-              g_object_unref (toplevel_device);
-              g_object_unref (toplevel_presentable);
+          if (toplevel_drive != NULL)
+            {
+              if (GDU_IS_DRIVE (toplevel_drive))
+                {
+                  GduDevice *toplevel_drive_device;
+
+                  drive = NULL;
+                  toplevel_drive_device = gdu_presentable_get_device (toplevel_drive);
+                  if (toplevel_drive_device != NULL)
+                    {
+                      drive = find_drive_by_device_file (monitor, gdu_device_get_device_file (toplevel_drive_device));
+                      /*g_debug ("adding volume %s (drive %s)",
+                        gdu_device_get_device_file (d),
+                        gdu_device_get_device_file (toplevel_device));*/
+                      g_object_unref (toplevel_drive_device);
+                    }
+                }
+              g_object_unref (toplevel_drive);
             }
           else
             {
@@ -1331,9 +1387,10 @@ update_volumes (GGduVolumeMonitor *monitor,
               monitor->volumes = g_list_prepend (monitor->volumes, volume);
               *added_volumes = g_list_prepend (*added_volumes, g_object_ref (volume));
             }
-        }
+         }
 
-      g_object_unref (d);
+       if (d != NULL)
+         g_object_unref (d);
     }
 
   g_list_free (added);
@@ -1510,7 +1567,7 @@ update_mounts (GGduVolumeMonitor *monitor,
       if (volume == NULL)
         volume = find_volume_for_mount_path (monitor, mount_path);
 
-      /*g_debug ("adding mount %s (vol %p)", g_unix_mount_get_device_path (mount_entry), volume);*/
+      /*g_debug ("adding mount %s (vol %p) (device %s, mount_path %s)", g_unix_mount_get_device_path (mount_entry), volume, device_file, mount_path);*/
       mount = g_gdu_mount_new (G_VOLUME_MONITOR (monitor), mount_entry, volume);
       if (mount)
         {
@@ -1590,10 +1647,15 @@ update_discs (GGduVolumeMonitor *monitor,
       GduPresentable *p = GDU_PRESENTABLE (l->data);
       GduDevice *d;
 
+      volume = NULL;
+      mount = NULL;
       d = gdu_presentable_get_device (p);
 
-      volume = find_disc_volume_for_device_file (monitor, gdu_device_get_device_file (d));
-      mount = find_disc_mount_for_volume (monitor, volume);
+      if (d != NULL)
+        {
+          volume = find_disc_volume_for_device_file (monitor, gdu_device_get_device_file (d));
+          mount = find_disc_mount_for_volume (monitor, volume);
+        }
 
       if (mount != NULL)
         {
@@ -1611,7 +1673,8 @@ update_discs (GGduVolumeMonitor *monitor,
           *removed_volumes = g_list_prepend (*removed_volumes, volume);
         }
 
-      g_object_unref (d);
+      if (d != NULL)
+        g_object_unref (d);
     }
 
   for (l = added; l != NULL; l = l->next)
@@ -1620,27 +1683,48 @@ update_discs (GGduVolumeMonitor *monitor,
       GduDevice *d;
       gboolean is_blank;
 
+      volume = NULL;
+      is_blank = TRUE;
       d = gdu_presentable_get_device (p);
 
-      is_blank = gdu_device_optical_disc_get_is_blank (d);
+      if (d != NULL)
+        {
+          is_blank = gdu_device_optical_disc_get_is_blank (d);
+          volume = find_disc_volume_for_device_file (monitor, gdu_device_get_device_file (d));
+        }
 
-      volume = find_disc_volume_for_device_file (monitor, gdu_device_get_device_file (d));
       if (volume == NULL)
         {
-          GduPresentable *toplevel_presentable;
+          GduPresentable *toplevel_drive;
 
-          toplevel_presentable = gdu_presentable_get_toplevel (p);
-          if (toplevel_presentable != NULL)
+          toplevel_drive = gdu_presentable_get_enclosing_presentable (p);
+          /* handle logical partitions enclosed by an extented partition */
+          if (GDU_IS_VOLUME (toplevel_drive))
             {
-              GduDevice *toplevel_device;
+              GduPresentable *temp;
+              temp = toplevel_drive;
+              toplevel_drive = gdu_presentable_get_enclosing_presentable (toplevel_drive);
+              g_object_unref (temp);
+            }
 
-              toplevel_device = gdu_presentable_get_device (toplevel_presentable);
-              drive = find_drive_by_device_file (monitor, gdu_device_get_device_file (toplevel_device));
-              /*g_debug ("adding volume %s (drive %s)",
-                       gdu_device_get_device_file (d),
-                       gdu_device_get_device_file (toplevel_device));*/
-              g_object_unref (toplevel_device);
-              g_object_unref (toplevel_presentable);
+          if (toplevel_drive != NULL)
+            {
+              if (GDU_IS_DRIVE (toplevel_drive))
+                {
+                  GduDevice *toplevel_drive_device;
+
+                  drive = NULL;
+                  toplevel_drive_device = gdu_presentable_get_device (toplevel_drive);
+                  if (toplevel_drive_device != NULL)
+                    {
+                      drive = find_drive_by_device_file (monitor, gdu_device_get_device_file (toplevel_drive_device));
+                      /*g_debug ("adding volume %s (drive %s)",
+                        gdu_device_get_device_file (d),
+                        gdu_device_get_device_file (toplevel_device));*/
+                      g_object_unref (toplevel_drive_device);
+                    }
+                }
+              g_object_unref (toplevel_drive);
             }
           else
             {
@@ -1693,7 +1777,8 @@ update_discs (GGduVolumeMonitor *monitor,
             }
         }
 
-      g_object_unref (d);
+      if (d != NULL)
+        g_object_unref (d);
     }
 
   g_list_free (added);
